/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.interfaces;

import java.io.IOException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.command.CommandStackListener;
import org.eclipse.emf.ecore.EObject;

/**
 * Interface for accessing top-level elements provided by
 * {@link IStorageProvider}s.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 95B275B63B2E2FC389D1F8339E2325C9
 */
public interface ITopLevelElementContext {

	/** Returns the top-level model element represented by this context. */
	EObject getTopLevelElement();

	/** Executes the given {@link Runnable} as model changing command-. */
	void runAsCommand(Runnable runner);

	/** Adds a command stack listener. */
	void addCommandStackListener(CommandStackListener listener);

	/** Removes the command stack listener. */
	void removeCommandStackListener(CommandStackListener listener);

	/** Returns whether undo operation is possible. */
	boolean canUndo();

	/** Returns whether redo operation is possible. */
	boolean canRedo();

	/** Executes the undo operation. */
	void undo();

	/** Executes the redo operation. */
	void redo();

	/** Returns whether there is unsaved content. */
	boolean isDirty();

	/** Returns the name of the saveable unit. */
	String getSaveableName();

	/** Performs the save operation. */
	void doSave(IProgressMonitor monitor) throws CoreException, IOException;

}
