/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.services;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.interfaces.IStorageProvider;
import org.fortiss.tooling.kernel.interfaces.ITopLevelElementChangeListener;
import org.fortiss.tooling.kernel.interfaces.ITopLevelElementContext;
import org.fortiss.tooling.kernel.internal.PersistencyService;

/**
 * The persistency service provides the transparent access to the different
 * persistency options, namely an EMFStore or local XML files within an Eclipse
 * project. Independent of the concrete {@link IStorageProvider} variant the
 * persistency service offers a list of {@link ITopLevelElementContext}s and a
 * notification mechanism for changes to this list.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 40AD35D9D289A3538D489B03405B2CD3
 */
public interface IPersistencyService {

	/** Returns the singleton instance of the service. */
	public static final IPersistencyService INSTANCE = new PersistencyService();

	/**
	 * Returns the list of top level {@link ITopLevelElementContext}s provided
	 * by {@link IStorageProvider}s.
	 */
	List<ITopLevelElementContext> getTopLevelElementContexts();

	/** Returns whether the given element is a top-level element. */
	boolean isTopLevelElement(EObject element);

	/** Adds a top-level element listener. */
	void addTopLevelElementListener(ITopLevelElementChangeListener listener);

	/** Removes a top-level element listener. */
	void removeTopLevelElementListener(ITopLevelElementChangeListener listener);

	/** Returns the top-level element for the given model element. */
	ITopLevelElementContext getTopLevelElementContextFor(EObject modelElement);

	/** Returns whether the some storage provider has unsaved changes. */
	boolean isDirty();

	/** Saves any dirty top-level elements to disk. */
	void doSave(IProgressMonitor monitor);

	/**
	 * Refreshes the list of top-level elements. This method should be called by
	 * storage providers whenever their list of provided top-level elements
	 * changes.
	 */
	void refreshTopLevelElements(IStorageProvider provider);
}
